@@ -121,7 +121,11 @@ xorg_lock_smasher_set_active (GSGrab  *grab,
 	status = XF86MiscSetGrabKeysState (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), active);
 
 	gdk_display_sync (gdk_display_get_default ());
+#if GTK_CHECK_VERSION (3, 0, 0)
+	error = gdk_error_trap_pop ();
+#else
 	gdk_error_trap_pop ();
+#endif
 
 	if (active && status == MiscExtGrabStateAlready)
 	{
@@ -129,12 +133,26 @@ xorg_lock_smasher_set_active (GSGrab  *grab,
 		status = MiscExtGrabStateSuccess;
 	}
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+        if (error == Success) {
+                gs_debug ("XF86MiscSetGrabKeysState(%s) returned %s\n",
+                          active ? "on" : "off",
+                          (status == MiscExtGrabStateSuccess ? "MiscExtGrabStateSuccess" :
+                           status == MiscExtGrabStateLocked  ? "MiscExtGrabStateLocked"  :
+                           status == MiscExtGrabStateAlready ? "MiscExtGrabStateAlready" :
+                           "unknown value"));
+        } else {
+                gs_debug ("XF86MiscSetGrabKeysState(%s) failed with error code %d\n",
+                          active ? "on" : "off", error);
+        }
+#else
 	gs_debug ("XF86MiscSetGrabKeysState(%s) returned %s\n",
 	          active ? "on" : "off",
 	          (status == MiscExtGrabStateSuccess ? "MiscExtGrabStateSuccess" :
 	           status == MiscExtGrabStateLocked  ? "MiscExtGrabStateLocked"  :
 	           status == MiscExtGrabStateAlready ? "MiscExtGrabStateAlready" :
 	           "unknown value"));
+#endif
 }
 #else
 static void
@@ -186,12 +204,20 @@ gs_grab_get_mouse (GSGrab    *grab,
                    gboolean   hide_cursor)
 {
 	GdkGrabStatus status;
+#if GTK_CHECK_VERSION (3, 16, 0)
+	GdkDisplay *display;
+#endif
 	GdkCursor    *cursor;
 
 	g_return_val_if_fail (window != NULL, FALSE);
 	g_return_val_if_fail (screen != NULL, FALSE);
 
+#if GTK_CHECK_VERSION (3, 16, 0)
+	display = gtk_widget_get_display (GTK_WIDGET (window));
+	cursor = gdk_cursor_new_for_display (display, GDK_BLANK_CURSOR);
+#else
 	cursor = gdk_cursor_new (GDK_BLANK_CURSOR);
+#endif
 
 	gs_debug ("Grabbing mouse widget=%X", (guint32) GDK_WINDOW_XID (window));
 	status = gdk_pointer_grab (window, TRUE, 0, NULL,
@@ -214,7 +240,11 @@ gs_grab_get_mouse (GSGrab    *grab,
 		grab->priv->mouse_hide_cursor = hide_cursor;
 	}
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	g_object_unref (cursor);
+#else
 	gdk_cursor_unref (cursor);
+#endif
 
 	return status;
 }
@@ -416,8 +446,12 @@ gs_grab_nuke_focus (void)
 	XGetInputFocus (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), &focus, &rev);
 	XSetInputFocus (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), None, RevertToNone, CurrentTime);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gdk_error_trap_pop_ignored ();
+#else
 	gdk_display_sync (gdk_display_get_default ());
 	gdk_error_trap_pop ();
+#endif
 }
 
 void
