@@ -38,11 +38,11 @@
 #include <gtk/gtk.h>
 #include <gio/gio.h>
 
-#if GTK_CHECK_VERSION (3, 0, 0)
-#include <gtk/gtkx.h>
 #define MATE_DESKTOP_USE_UNSTABLE_API
 #include <libmate-desktop/mate-desktop-utils.h>
-#define gdk_spawn_command_line_on_screen mate_gdk_spawn_command_line_on_screen
+
+#if GTK_CHECK_VERSION (3, 0, 0)
+#include <gtk/gtkx.h>
 #endif
 
 #ifdef WITH_KBD_LAYOUT_INDICATOR
@@ -82,6 +82,11 @@ static void gs_lock_plug_finalize   (GObject         *object);
 
 #define GS_LOCK_PLUG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GS_TYPE_LOCK_PLUG, GSLockPlugPrivate))
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+#define gtk_hbox_new(X,Y) gtk_box_new(GTK_ORIENTATION_HORIZONTAL,Y)
+#define gtk_vbox_new(X,Y) gtk_box_new(GTK_ORIENTATION_VERTICAL,Y)
+#endif
+
 struct GSLockPlugPrivate
 {
 	GtkWidget   *vbox;
@@ -215,7 +220,7 @@ do_user_switch (GSLockPlug *plug)
 								   MDM_FLEXISERVER_ARGS);
 
 		error = NULL;
-		res = gdk_spawn_command_line_on_screen (gdk_screen_get_default (),
+		res = mate_gdk_spawn_command_line_on_screen (gdk_screen_get_default (),
 												command,
 												&error);
 
@@ -235,7 +240,7 @@ do_user_switch (GSLockPlug *plug)
 								   GDM_FLEXISERVER_ARGS);
 
 		error = NULL;
-		res = gdk_spawn_command_line_on_screen (gdk_screen_get_default (),
+		res = mate_gdk_spawn_command_line_on_screen (gdk_screen_get_default (),
 												command,
 												&error);
 
@@ -623,9 +628,13 @@ gs_lock_plug_run (GSLockPlug *plug)
 
 	ri.loop = g_main_loop_new (NULL, FALSE);
 
+#if !GTK_CHECK_VERSION (3, 0, 0)
 	GDK_THREADS_LEAVE ();
+#endif
 	g_main_loop_run (ri.loop);
+#if !GTK_CHECK_VERSION (3, 0, 0)
 	GDK_THREADS_ENTER ();
+#endif
 
 	g_main_loop_unref (ri.loop);
 
@@ -1547,28 +1556,52 @@ logout_button_clicked (GtkButton  *button,
 void
 gs_lock_plug_set_busy (GSLockPlug *plug)
 {
+#if GTK_CHECK_VERSION (3, 16, 0)
+	GdkDisplay *display;
+#endif
 	GdkCursor *cursor;
 	GtkWidget *top_level;
 
 	top_level = gtk_widget_get_toplevel (GTK_WIDGET (plug));
 
+#if GTK_CHECK_VERSION (3, 16, 0)
+	display = gtk_widget_get_display (GTK_WIDGET (plug));
+	cursor = gdk_cursor_new_for_display (display, GDK_WATCH);
+#else
 	cursor = gdk_cursor_new (GDK_WATCH);
+#endif
 
 	gdk_window_set_cursor (gtk_widget_get_window (top_level), cursor);
+#if GTK_CHECK_VERSION (3, 0, 0)
+	g_object_unref (cursor);
+#else
 	gdk_cursor_unref (cursor);
+#endif
 }
 
 void
 gs_lock_plug_set_ready (GSLockPlug *plug)
 {
+#if GTK_CHECK_VERSION (3, 16, 0)
+	GdkDisplay *display;
+#endif
 	GdkCursor *cursor;
 	GtkWidget *top_level;
 
 	top_level = gtk_widget_get_toplevel (GTK_WIDGET (plug));
 
+#if GTK_CHECK_VERSION (3, 16, 0)
+	display = gtk_widget_get_display (GTK_WIDGET (plug));
+	cursor = gdk_cursor_new_for_display (display, GDK_LEFT_PTR);
+#else
 	cursor = gdk_cursor_new (GDK_LEFT_PTR);
+#endif
 	gdk_window_set_cursor (gtk_widget_get_window (top_level), cursor);
+#if GTK_CHECK_VERSION (3, 0, 0)
+	g_object_unref (cursor);
+#else
 	gdk_cursor_unref (cursor);
+#endif
 }
 
 void
@@ -1927,20 +1960,35 @@ create_page_one (GSLockPlug *plug)
 	str = g_strdup ("<span size=\"xx-large\" weight=\"ultrabold\">%s</span>");
 	plug->priv->auth_time_label = gtk_label_new (str);
 	g_free (str);
+#if GTK_CHECK_VERSION (3, 16, 0)
+	gtk_label_set_xalign (GTK_LABEL (plug->priv->auth_time_label), 0.5);
+	gtk_label_set_xalign (GTK_LABEL (plug->priv->auth_time_label), 0.5);
+#else
 	gtk_misc_set_alignment (GTK_MISC (plug->priv->auth_time_label), 0.5, 0.5);
+#endif
 	gtk_label_set_use_markup (GTK_LABEL (plug->priv->auth_time_label), TRUE);
 	gtk_box_pack_start (GTK_BOX (vbox2), plug->priv->auth_time_label, FALSE, FALSE, 0);
 
 	str = g_strdup ("<span size=\"large\">%s</span>");
 	plug->priv->auth_date_label = gtk_label_new (str);
 	g_free (str);
+#if GTK_CHECK_VERSION (3, 16, 0)
+	gtk_label_set_xalign (GTK_LABEL (plug->priv->auth_date_label), 0.5);
+	gtk_label_set_xalign (GTK_LABEL (plug->priv->auth_date_label), 0.5);
+#else
 	gtk_misc_set_alignment (GTK_MISC (plug->priv->auth_date_label), 0.5, 0.5);
+#endif
 	gtk_label_set_use_markup (GTK_LABEL (plug->priv->auth_date_label), TRUE);
 	gtk_box_pack_start (GTK_BOX (vbox2), plug->priv->auth_date_label, FALSE, FALSE, 0);
 
 	plug->priv->auth_face_image = gtk_image_new ();
 	gtk_box_pack_start (GTK_BOX (vbox), plug->priv->auth_face_image, TRUE, TRUE, 0);
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gtk_widget_set_halign (plug->priv->auth_face_image, GTK_ALIGN_CENTER);
+	gtk_widget_set_valign (plug->priv->auth_face_image, GTK_ALIGN_END);
+#else
 	gtk_misc_set_alignment (GTK_MISC (plug->priv->auth_face_image), 0.5, 1.0);
+#endif
 
 	vbox2 = gtk_vbox_new (FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (vbox), vbox2, FALSE, FALSE, 0);
@@ -1949,7 +1997,12 @@ create_page_one (GSLockPlug *plug)
 	plug->priv->auth_realname_label = gtk_label_new (str);
 	g_free (str);
 	expand_string_for_label (plug->priv->auth_realname_label);
+#if GTK_CHECK_VERSION (3, 16, 0)
+	gtk_label_set_xalign (GTK_LABEL (plug->priv->auth_realname_label), 0.5);
+	gtk_label_set_xalign (GTK_LABEL (plug->priv->auth_realname_label), 0.5);
+#else
 	gtk_misc_set_alignment (GTK_MISC (plug->priv->auth_realname_label), 0.5, 0.5);
+#endif
 	gtk_label_set_use_markup (GTK_LABEL (plug->priv->auth_realname_label), TRUE);
 	gtk_box_pack_start (GTK_BOX (vbox2), plug->priv->auth_realname_label, FALSE, FALSE, 0);
 
@@ -1958,7 +2011,12 @@ create_page_one (GSLockPlug *plug)
 	plug->priv->auth_username_label = gtk_label_new (str);
 	g_free (str);
 	expand_string_for_label (plug->priv->auth_username_label);
+#if GTK_CHECK_VERSION (3, 16, 0)
+	gtk_label_set_xalign (GTK_LABEL (plug->priv->auth_realname_label), 0.5);
+	gtk_label_set_xalign (GTK_LABEL (plug->priv->auth_realname_label), 0.5);
+#else
 	gtk_misc_set_alignment (GTK_MISC (plug->priv->auth_username_label), 0.5, 0.5);
+#endif
 	gtk_label_set_use_markup (GTK_LABEL (plug->priv->auth_username_label), TRUE);
 	gtk_box_pack_start (GTK_BOX (vbox2), plug->priv->auth_username_label, FALSE, FALSE, 0);
 
@@ -1969,7 +2027,11 @@ create_page_one (GSLockPlug *plug)
 	gtk_box_pack_start (GTK_BOX (vbox2), hbox, FALSE, FALSE, 0);
 
 	plug->priv->auth_prompt_label = gtk_label_new_with_mnemonic (_("_Password:"));
-	gtk_misc_set_alignment (GTK_MISC (plug->priv->auth_prompt_label), 0, 0.5);
+#if GTK_CHECK_VERSION (3, 16, 0)
+	gtk_label_set_xalign (GTK_LABEL (plug->priv->auth_prompt_label), 0.0);
+#else
+	gtk_misc_set_alignment (GTK_MISC (plug->priv->auth_prompt_label), 0.0, 0.5);
+#endif
 	gtk_box_pack_start (GTK_BOX (hbox), plug->priv->auth_prompt_label, FALSE, FALSE, 0);
 
 	plug->priv->auth_prompt_entry = gtk_entry_new ();
@@ -1979,7 +2041,12 @@ create_page_one (GSLockPlug *plug)
 	                               plug->priv->auth_prompt_entry);
 
 	plug->priv->auth_capslock_label = gtk_label_new ("");
+#if GTK_CHECK_VERSION (3, 16, 0)
+	gtk_label_set_xalign (GTK_LABEL (plug->priv->auth_capslock_label), 0.5);
+	gtk_label_set_xalign (GTK_LABEL (plug->priv->auth_capslock_label), 0.5);
+#else
 	gtk_misc_set_alignment (GTK_MISC (plug->priv->auth_capslock_label), 0.5, 0.5);
+#endif
 	gtk_box_pack_start (GTK_BOX (vbox2), plug->priv->auth_capslock_label, FALSE, FALSE, 0);
 
 	/* Status text */
@@ -1988,7 +2055,11 @@ create_page_one (GSLockPlug *plug)
 	gtk_box_pack_start (GTK_BOX (vbox), plug->priv->auth_message_label,
 	                    FALSE, FALSE, 0);
 	/* Buttons */
+#if GTK_CHECK_VERSION(3, 0, 0)
+	plug->priv->auth_action_area = gtk_button_box_new (GTK_ORIENTATION_HORIZONTAL);
+#else
 	plug->priv->auth_action_area = gtk_hbutton_box_new ();
+#endif
 
 	gtk_button_box_set_layout (GTK_BUTTON_BOX (plug->priv->auth_action_area),
 	                           GTK_BUTTONBOX_END);
